@@ -1,9 +1,9 @@
-use anyhow::{bail, Error};
+use anyhow::{Error, bail};
 use pest::iterators::Pair;
 
 use document_tree::{
-    attribute_types as at, element_categories as c, elements as e, extra_attributes as a, Element,
-    ExtraAttributes, HasChildren,
+    Element, ExtraAttributes, HasChildren, attribute_types as at, element_categories as c,
+    elements as e, extra_attributes as a,
 };
 
 use super::{inline::convert_inlines, whitespace_normalize_name};
@@ -21,7 +21,7 @@ pub(super) enum TitleOrSsubel {
 }
 
 pub(super) fn convert_ssubel(pair: Pair<Rule>) -> Result<Option<TitleOrSsubel>, Error> {
-    use self::TitleOrSsubel::*;
+    use self::TitleOrSsubel::{Ssubel, Title};
     Ok(Some(match pair.as_rule() {
         Rule::title => {
             let (t, k) = convert_title(pair)?;
@@ -46,10 +46,13 @@ fn convert_body_elem(pair: Pair<Rule>) -> Result<c::BodyElement, Error> {
     Ok(match pair.as_rule() {
         Rule::paragraph => convert_paragraph(pair)?.into(),
         Rule::target => convert_target(pair)?.into(),
+        Rule::footnote => convert_footnote(pair)?.into(),
         Rule::substitution_def => convert_substitution_def(pair)?.into(),
-        Rule::admonition_gen => convert_admonition_gen(pair)?,
+        Rule::block_quote_directive => convert_block_quote_directive(pair)?.into(),
+        Rule::admonition_gen => convert_admonition_gen(pair),
         Rule::image => convert_image::<e::Image>(pair)?.into(),
         Rule::bullet_list => convert_bullet_list(pair)?.into(),
+        Rule::block_quote => convert_block_quote(pair)?.into(),
         Rule::literal_block => convert_literal_block(pair).into(),
         Rule::code_directive => convert_code_directive(pair).into(),
         Rule::raw_directive => convert_raw_directive(pair).into(),
@@ -72,10 +75,10 @@ fn convert_title(pair: Pair<Rule>) -> Result<(e::Title, TitleKind), Error> {
                 title_inlines = Some(convert_inlines(p)?);
             }
             Rule::adornments => {
-                adornment_char = Some(p.as_str().chars().next().expect("Empty adornment?"))
+                adornment_char = Some(p.as_str().chars().next().expect("Empty adornment?"));
             }
             rule => unimplemented!("Unexpected rule in title: {:?}", rule),
-        };
+        }
     }
     // now we encountered one line of text and one of adornments
     // TODO: emit error if the adornment line is too short (has to match title length)
@@ -98,7 +101,7 @@ fn convert_paragraph(pair: Pair<Rule>) -> Result<e::Paragraph, Error> {
 }
 
 fn convert_target(pair: Pair<Rule>) -> Result<e::Target, Error> {
-    let mut elem: e::Target = Default::default();
+    let mut elem = e::Target::default();
     elem.extra_mut().anonymous = false;
     for p in pair.into_inner() {
         match p.as_rule() {
@@ -108,12 +111,43 @@ fn convert_target(pair: Pair<Rule>) -> Result<e::Target, Error> {
             }
             // TODO: also handle non-urls
             Rule::link_target => elem.extra_mut().refuri = Some(p.parse()?),
-            rule => panic!("Unexpected rule in target: {:?}", rule),
+            rule => panic!("Unexpected rule in target: {rule:?}"),
         }
     }
     Ok(elem)
 }
 
+/// Converts a footnote.
+/// - named auto-numbered footnotes get their name set
+/// - explicitly numbered footnotes get their label set
+fn convert_footnote(pair: Pair<Rule>) -> Result<e::Footnote, Error> {
+    let mut pairs = pair.into_inner();
+    let label = pairs.next().unwrap().as_str();
+    let mut children: Vec<c::SubFootnote> = vec![];
+    // turn `line` into paragraph
+    children.push(convert_paragraph(pairs.next().unwrap())?.into());
+    for p in pairs {
+        children.push(convert_body_elem(p)?.into());
+    }
+    let mut footnote = e::Footnote::with_children(children);
+    footnote.extra_mut().auto = label.chars().next().unwrap().try_into().ok();
+    match footnote.extra().auto {
+        Some(at::AutoFootnoteType::Number) => {
+            if label.len() > 1 {
+                let name = whitespace_normalize_name(&label[1..]);
+                footnote.names_mut().push(at::NameToken(name));
+            }
+        }
+        Some(at::AutoFootnoteType::Symbol) => {}
+        None => {
+            footnote
+                .children_mut()
+                .insert(0, e::Label::with_children(vec![label.into()]).into());
+        }
+    }
+    Ok(footnote)
+}
+
 fn convert_substitution_def(pair: Pair<Rule>) -> Result<e::SubstitutionDefinition, Error> {
     let mut pairs = pair.into_inner();
     let name = whitespace_normalize_name(pairs.next().unwrap().as_str()); // Rule::substitution_name
@@ -121,7 +155,7 @@ fn convert_substitution_def(pair: Pair<Rule>) -> Result<e::SubstitutionDefinitio
     let inner: Vec<c::TextOrInlineElement> = match inner_pair.as_rule() {
         Rule::replace => convert_replace(inner_pair)?,
         Rule::image => vec![convert_image::<e::ImageInline>(inner_pair)?.into()],
-        rule => panic!("Unknown substitution rule {:?}", rule),
+        rule => panic!("Unknown substitution rule {rule:?}"),
     };
     let mut subst_def = e::SubstitutionDefinition::with_children(inner);
     subst_def.names_mut().push(at::NameToken(name));
@@ -162,13 +196,14 @@ where
 }
 
 fn parse_scale(pair: &Pair<Rule>) -> Result<u8, Error> {
+    use pest::error::{Error, ErrorVariant};
+
     let input = pair.as_str().trim();
     let input = if let Some(percentage) = input.strip_suffix('%') {
         percentage.trim_end()
     } else {
         input
     };
-    use pest::error::{Error, ErrorVariant};
     Ok(input.parse().map_err(|e: std::num::ParseIntError| {
         let var: ErrorVariant<Rule> = ErrorVariant::CustomError {
             message: e.to_string(),
@@ -177,14 +212,14 @@ fn parse_scale(pair: &Pair<Rule>) -> Result<u8, Error> {
     })?)
 }
 
-fn convert_admonition_gen(pair: Pair<Rule>) -> Result<c::BodyElement, Error> {
+fn convert_admonition_gen(pair: Pair<Rule>) -> document_tree::element_categories::BodyElement {
     let mut iter = pair.into_inner();
     let typ = iter.next().unwrap().as_str();
     // TODO: in reality it contains body elements.
     let children: Vec<c::BodyElement> = iter
         .map(|p| e::Paragraph::with_children(vec![p.as_str().into()]).into())
         .collect();
-    Ok(match typ {
+    match typ {
         "attention" => e::Attention::with_children(children).into(),
         "hint" => e::Hint::with_children(children).into(),
         "note" => e::Note::with_children(children).into(),
@@ -194,8 +229,8 @@ fn convert_admonition_gen(pair: Pair<Rule>) -> Result<c::BodyElement, Error> {
         "important" => e::Important::with_children(children).into(),
         "tip" => e::Tip::with_children(children).into(),
         "warning" => e::Warning::with_children(children).into(),
-        typ => panic!("Unknown admontion type {}!", typ),
-    })
+        typ => panic!("Unknown admontion type {typ}!"),
+    }
 }
 
 fn convert_bullet_list(pair: Pair<Rule>) -> Result<e::BulletList, Error> {
@@ -215,6 +250,33 @@ fn convert_bullet_item(pair: Pair<Rule>) -> Result<e::ListItem, Error> {
     Ok(e::ListItem::with_children(children))
 }
 
+fn convert_block_quote(pair: Pair<Rule>) -> Result<e::BlockQuote, Error> {
+    Ok(e::BlockQuote::with_children(
+        pair.into_inner()
+            .map(convert_block_quote_inner)
+            .collect::<Result<_, _>>()?,
+    ))
+}
+
+fn convert_block_quote_directive(pair: Pair<Rule>) -> Result<e::BlockQuote, Error> {
+    let mut iter = pair.into_inner();
+    let typ = iter.next().unwrap().as_str();
+    let children: Vec<c::SubBlockQuote> = iter
+        .map(convert_block_quote_inner)
+        .collect::<Result<_, _>>()?;
+    let mut bq = e::BlockQuote::with_children(children);
+    bq.classes_mut().push(typ.to_owned());
+    Ok(bq)
+}
+
+fn convert_block_quote_inner(pair: Pair<Rule>) -> Result<c::SubBlockQuote, Error> {
+    Ok(if pair.as_rule() == Rule::attribution {
+        e::Attribution::with_children(convert_inlines(pair)?).into()
+    } else {
+        convert_body_elem(pair)?.into()
+    })
+}
+
 fn convert_literal_block(pair: Pair<Rule>) -> e::LiteralBlock {
     convert_literal_lines(pair.into_inner().next().unwrap())
 }
@@ -244,7 +306,7 @@ fn convert_code_directive(pair: Pair<Rule>) -> e::LiteralBlock {
     code_block.classes_mut().push("code".to_owned());
     if let Some(lang) = lang {
         code_block.classes_mut().push(lang.as_str().to_owned());
-    };
+    }
     code_block
 }
 
