@@ -44,6 +44,8 @@ fn emph() {
     };
 }
 
+// TODO: escaped chars
+
 #[test]
 fn title() {
     parses_to! {
@@ -104,6 +106,71 @@ fn two_targets() {
     };
 }
 
+#[test]
+fn footnote() {
+    parses_to! {
+        parser: RstParser,
+        input: ".. [#] A\n",
+        rule: Rule::footnote,
+        tokens: [
+            footnote(0, 9, [
+                footnote_label(4, 5),
+                line(7, 9, [ str(7, 8) ]),
+            ]),
+        ]
+    };
+}
+
+#[test]
+fn footnotes() {
+    parses_to! {
+        parser: RstParser,
+        input: "\
+.. [#] A
+.. [#b] B
+   More
+
+.. [32] C
+",
+        rule: Rule::document,
+        tokens: [
+            footnote(0, 9, [
+                footnote_label(4, 5),
+                line(7, 9, [ str(7, 8) ]),
+            ]),
+            footnote(9, 28, [
+                footnote_label(13, 15),
+                line(17, 19, [ str(17, 18) ]),
+                paragraph(22, 26, [ str(22, 26) ]),
+            ]),
+            footnote(28, 38, [
+                footnote_label(32, 34),
+                line(36, 38, [ str(36, 37) ]),
+            ]),
+        ]
+    };
+}
+
+#[test]
+fn footnote_refs() {
+    parses_to! {
+        parser: RstParser,
+        input: "A-[#one]_ and [#]_ and [*]_.",
+        rule: Rule::document,
+        tokens: [
+            paragraph(0, 28, [
+                str(0, 2),
+                footnote_reference(2, 9, [ footnote_label(3, 7) ]),
+                str(9, 14),
+                footnote_reference(14, 18, [ footnote_label(15, 16) ]),
+                str(18, 23),
+                footnote_reference(23, 27, [ footnote_label(24, 25) ]),
+                str(27, 28),
+            ]),
+        ]
+    };
+}
+
 #[test]
 fn inline_code_literal_with_underscore() {
     parses_to! {
@@ -132,11 +199,11 @@ fn admonitions() {
 ",
         rule: Rule::document,
         tokens: [
-            admonition_gen(0, 27, [
+            admonition_gen(0, 28, [
                 admonition_type(3, 7),
                 paragraph(13, 27, [ str(13, 27) ]),
             ]),
-            admonition(28, 71, [
+            admonition(28, 73, [
                 line(43, 58, [ str(43, 57) ]),
                 paragraph(62, 71, [ str(62, 71) ]),
             ]),
@@ -148,6 +215,191 @@ fn admonitions() {
     };
 }
 
+#[test]
+fn blockquote_no_attribution() {
+    parses_to! {
+        parser: RstParser,
+        input: "\
+Paragraph.
+
+   Block quote.
+
+Paragraph.
+",
+        rule: Rule::document,
+        tokens: [
+            paragraph(0, 10, [ str(0, 10) ]),
+            block_quote(12, 29, [
+                paragraph(15, 27, [ str(15, 27) ]),
+            ]),
+            paragraph(29, 39, [ str(29, 39) ]),
+        ]
+    };
+}
+
+#[test]
+fn blockquote_fake_attribution() {
+    parses_to! {
+        parser: RstParser,
+        input: "\
+Paragraph.
+
+   -- Not an attribution
+
+Paragraph.
+",
+        rule: Rule::document,
+        tokens: [
+            paragraph(0, 10, [ str(0, 10) ]),
+            block_quote(12, 38, [
+                paragraph(15, 36, [ str(15, 36) ]),
+            ]),
+            paragraph(38, 48, [ str(38, 48) ]),
+        ]
+    };
+}
+
+#[test]
+fn blockquote_simple() {
+    parses_to! {
+        parser: RstParser,
+        input: "\
+Paragraph.
+
+   Block quote.
+
+   -- Attribution
+
+Paragraph.
+",
+        rule: Rule::document,
+        tokens: [
+            paragraph(0, 10, [ str(0, 10) ]),
+            block_quote(12, 47, [
+                paragraph(15, 27, [ str(15, 27) ]),
+                attribution(29, 47, [ line(35, 47, [ str(35, 46) ]) ]),
+            ]),
+            paragraph(48, 58, [ str(48, 58) ]),
+        ]
+    };
+}
+
+#[test]
+fn blockquote_multiline_attribution() {
+    parses_to! {
+        parser: RstParser,
+        input: "\
+Paragraph.
+
+   Block quote.
+
+   --Attribution line one
+   and line two
+
+Paragraph.
+",
+        rule: Rule::document,
+        tokens: [
+            paragraph(0, 10, [ str(0, 10) ]),
+            block_quote(12, 71, [
+                paragraph(15, 27, [ str(15, 27) ]),
+                attribution(29, 71, [
+                    line(34, 55, [ str(34, 54) ]),
+                    line(55, 71, [ str(55, 70) ])
+                ]),
+            ]),
+            paragraph(72, 82, [ str(72, 82) ]),
+        ]
+    };
+}
+
+#[test]
+fn blockquote_multiline_attribution_2() {
+    parses_to! {
+        parser: RstParser,
+        input: "\
+Paragraph.
+
+   -- Invalid attribution
+
+   Block quote.
+
+   -- Attribution line one
+      and line two
+
+Paragraph.
+",
+        rule: Rule::document,
+        tokens: [
+            paragraph(0, 10, [ str(0, 10) ]),
+            block_quote(12, 102, [
+                paragraph(15, 37, [ str(15, 37) ]),
+                paragraph(42, 54, [ str(42, 54) ]),
+                attribution(56, 102, [
+                    line(62, 83, [ str(62, 82) ]),
+                    line(83, 102, [ str(83, 101) ])
+                ])
+            ]),
+            paragraph(103, 113, [ str(103, 113) ]),
+        ]
+    };
+}
+
+#[test]
+fn blockquote_back_to_back() {
+    parses_to! {
+        parser: RstParser,
+        input: "\
+Paragraph.
+
+   Block quote 1.
+
+   -- Attribution 1
+
+   Block quote 2.
+
+   --Attribution 2
+
+Paragraph.
+",
+        rule: Rule::document,
+        tokens: [
+            paragraph(0, 10, [ str(0, 10) ]),
+            block_quote(12, 51, [
+                paragraph(15, 29, [ str(15, 29) ]),
+                attribution(31, 51, [ line(37, 51, [ str(37, 50) ]) ]),
+            ]),
+            block_quote(52, 90, [
+                paragraph(55, 69, [ str(55, 69) ]),
+                attribution(71, 90, [ line(76, 90, [ str(76, 89) ]) ]),
+            ]),
+            paragraph(91, 101, [ str(91, 101) ]),
+        ]
+    };
+}
+#[test]
+
+fn block_quote_directive() {
+    parses_to! {
+        parser: RstParser,
+        input: "\
+.. epigraph::
+
+   Single line
+
+The end
+",
+        rule: Rule::document,
+        tokens: [
+            block_quote_directive(0, 31, [
+                block_quote_type(3, 11),
+                paragraph(18, 29, [ str(18, 29) ]),
+            ]),
+            paragraph(31, 38, [ str(31, 38) ]),
+        ]
+    };
+}
+
 #[allow(clippy::cognitive_complexity)]
 #[test]
 fn literal_block() {
